#delimit ;
program define somersd, eclass byable(onecall) sortpreserve prop(svyj);
version 9.0;
/*
 Take, as input, 2 or more variables in varlist,
 comprising 1 x-variate and 1 or more y-variates,
 and, optionally, a flag asking for Kendall's tau-a instead of Somers' D.
 Create, as output, a maximum-likelihood mlout structure,
 with the vector of estimates equal to the vector of Somers' D's
 (or Kendall's tau-a's)
 of each y-variate with respect to the x-variate,
 and the vce matrix equal to their jackknife variance-covariance estimates.
 This program allows fweights, iweights or pweights.
*! Author: Roger Newson
*! Date: 11 August 2005
*/

if(replay()){;
*
 Beginning of replay section (not indented)
*;

if "`e(cmd)'"!="somersd"{;error 301;};
if _by() {;error 190;};
syntax [, Level(cilevel) CImatrix(passthru)];

*
 Display output
*;
_somersdplay, level(`level') `cimatrix';

*
 End of replay section (not indented)
*;
};
else{;
*
 Beginning of non-replay section (not indented)
*;

syntax varlist(min=2 max=9999 numeric) [if] [in] [fweight iweight pweight]
  [,
  TAua TDist TRansf(string)
  CEnind(string) CLuster(varname) CFWeight(string asis)
  FUntype(string) WStrata(varlist) BStrata(string)
  noTREe
  Level(cilevel)
  CImatrix(passthru)
  ];
/*
taua indicates that Kendall's tau-a is to be estimated instead of Somers' D
tdist specifies that we will assume a t-distribution
  for the Studentized estimate,
  with degrees of freedom equal to number of clusters minus one.
transf() specifies a transformation of Somers' D or Kendall's tau-a,
  for which estimates, standard errors and confidence limits are calculated.
cenind() specifies a list of left and/or right censorship indicators,
  which can be either varnames or zeros.
  Zero values indicate no censorship, negative values indicate left censorship,
  and positive values indicate right censorship.
cluster() specifies a clustering variable.
cfweight() specifies an expression for cluster frequency weights
  (used if each cluster represents multiple identical clusters).
funtype() specifies the functional type
  for the Somers' D and Kendall's tau-a parameters,
  and can be wcluster, bcluster or vonmises.
wstrata() specifies a list of variables whose combinations define the W-strata.
bstrata() specifies a list of variables whose combinations define the B-strata.
tree indicates that a search tree algorithm will be used.
level() specifies a confidence level.
cimatrix() specifies the name of a matrix to store the confidence limits
  (which may be asymmetric).
*/

*
 Set default tree algorithm setting
 and evaluate tree algorithm indicator
*;
local treeind="`tree'"!="notree";

*
 Initialise local macros to be used again and again
*;
local nvar: word count `varlist';
local nvarp=`nvar'+1;local nvarm=`nvar'-1;
local x: word 1 of `varlist';
* Y-variates (including the X-variate) *;
local i1=0;
while(`i1'<`nvar'){;local i1=`i1'+1;
  local y`i1': word `i1' of `varlist';
};
* List of y-variates (other than the X-variate) *;
local i1=1;local nxvlist "";
while(`i1'<`nvar'){;local i1=`i1'+1;
  local nxvlist "`nxvlist' `y`i1''";
};
* B-strata *;
local bstrata=trim(`"`bstrata'"');
if `"`bstrata'"'!="" & `"`bstrata'"'!="_n" {;
  cap confirm var `bstrata';
  if _rc!=0 {;
    disp as error "Invalid bstrata()";
    error 498;
  };
};
* Functional type *;
if `"`funtype'"'=="" {;local funtype="bcluster";};
local funtype=lower(`"`funtype'"');
foreach FT in wcluster bcluster vonmises {;
  if index("`FT'",`"`funtype'"')==1 {;
    local funtype="`FT'";
  };
};
if !inlist(`"`funtype'"',"bcluster","wcluster","vonmises") {;
  disp as error "Invalid funtype()";
  error 498;
};

*
 Evaluate weight expressions
*;
if "`_byvars'"!="" {;
  local bybyvars "by `_byvars' `_byrc0':";
};
tempvar wexpval cfwexpval;
* Ordinary weights *;
if `"`exp'"'=="" {;
  qui `bybyvars' gene byte `wexpval'=1;
};
else if "`weight'"=="fweight" {;
  qui `bybyvars' gene long `wexpval'`exp';
};
else {;
  qui `bybyvars' gene double `wexpval'`exp';
};
* Cluster frequency weights *;
if `"`cfweight'"'=="" {;
  qui `bybyvars' gene byte `cfwexpval'=1;
};
else {;
  cap qui `bybyvars' gene long `cfwexpval'=`cfweight';
  if _rc!=0 {;
    disp as error "Invalid cfweight()";
    error 498;
  };
};

* Create to-use variable *;
marksample touse;

*
 Create final list of censorship indicator variables
*;
local ncenind: word count `cenind';
if `ncenind'<`nvar' {;
  local ncenindp=`ncenind'+1;
  forv i1=`ncenindp'(1)`nvar' {;local cenind `"`cenind' 0"';};
};
tempvar zero;
qui gene byte `zero'=0 if `touse';
local cenindvars "";
forv i1=1(1)`nvar' {;
  local cenindcur: word `i1' of `cenind';
  cap confirm numeric variable `cenindcur';
  if _rc==0 {;
    local cenindvars "`cenindvars' `cenindcur'";
  };
  else if `"`cenindcur'"'=="0" {;
    local cenindvars "`cenindvars' `zero'";
  };
  else {;
    disp as error "Invalid option cenind()";
    error 498;
  };
};
local xcen: word 1 of `cenindvars';
* Exclude all observations with missing censorship indicators *;
foreach X of var `cenindvars' {;
  qui replace `touse'=0 if missing(`X');
};

*
 Define variables containing cluster frequency weights,
 observation frequency weights and importance weights.
 Importance weights are the w_hi in the formulae.
 Observation frequency weights are summed over all observations
 to evaluate the returned result e(N),
 and are used for nothing else.
 Cluster frequency weights (f_i for the i'th cluster)
 must be the same for all observations in the same cluster,
 and signify that the i'th cluster in the data set stands
 for f_i identical clusters in the true sample.
 If cfweight() is specified,
 then it is interpreted as cluster frequency weights,
 and fweights, iweights and pweights are all interpreted
 as importance weights.
 If cfweight() is unspecified and cluster() is specified,
 then fweights, iweights and pweights are all interpreted
 as importance weights,
 and cluster frequency weights are set to one.
 If cfweight() and cluster() are both unspecified,
 then fweights are interpreted as cluster frequency weights
 (and all importance weights are set to one),
 and iweights and pweights are interpreted as importance weights
 (and all cluster frequency weights are set to one).
 Therefore, if cweight() is unspecified,
 then this protocol is equivalent to the standard Stata practice
 of treating iweights and pweights as importance weights
 and treating fweights by doing the calculations
 as if there were multiple identical observations in the data set.
*;
tempvar cfwei ofwei iwei;
if `"`cfweight'"'!="" {;
  * Cluster frequency weights supplied by user *;
  qui gene long `cfwei'=`cfwexpval' if `touse';
  if "`weight'"=="" {;
    qui gene long `ofwei'=`cfwexpval' if `touse';
    qui gene byte `iwei'=1 if `touse';
  };
  else if "`weight'"=="fweight" {;
    qui gene long `ofwei'=`cfwexpval'*`wexpval' if `touse';
    qui gene long `iwei'=`wexpval' if `touse';
  };
  else {;
    qui gene long `ofwei'=`cfwexpval' if `touse';
    qui gene double `iwei'=`wexpval' if `touse';
  };
};
else if "`cluster'"!="" {;
  * Clusters specified without cfweights *;
  qui gene byte `cfwei'=1 if `touse';
  if "`weight'"=="" {;
    qui gene byte `ofwei'=1 if `touse';
    qui gene byte `iwei'=1 if `touse';
  };
  else if "`weight'"=="fweight" {;
    qui gene long `ofwei'=`wexpval' if `touse';
    qui gene long `iwei'=`wexpval' if `touse';
  };
  else {;
    qui gene byte `ofwei'=1 if `touse';
    qui gene double `iwei'=`wexpval' if `touse';
  };
};
else {;
  * No cfweights or clusters *;
  if "`weight'"=="" {;
    qui gene byte `cfwei'=1 if `touse';
    qui gene byte `ofwei'=1 if `touse';
    qui gene byte `iwei'=1 if `touse';
  };
  else if "`weight'"=="fweight" {;
    qui gene long `cfwei'=`wexpval' if `touse';
    qui gene long `ofwei'=`wexpval' if `touse';
    qui gene byte `iwei'=1 if `touse';
  };
  else {;
    qui gene byte `cfwei'=1 if `touse';
    qui gene byte `ofwei'=1 if `touse';
    qui gene double `iwei'=`wexpval' if `touse';
  };
};
* Exclude observations with missing or zero weights *;
foreach X of varlist `ofwei' `cfwei' `iwei' {;
  qui replace `touse'=0 if missing(`X') | (`X'==0);
};

*
 Generate product of weights
 (for use when calculating vidot and tidot variables)
*;
tempvar prodwei;
qui gene double `prodwei'=`cfwei'*`iwei' if `touse';

*
 Abort if all by-groups are empty
*;
qui summ `touse';
if r(sum)<=0 {;
  error 2000;
};

*
 Create by-group and cluster sequence variables
 and check that cluster frequency weights are constant within clusters
*;
tempvar bygp clseq cfwei2;
qui {;
  gsort `touse' `_byvars', gene(`bygp');
  if "`cluster'"=="" {;
    gsort `bygp' `_sortindex', gene(`clseq');
  };
  else {;
    gsort `bygp' `cluster', gene(`clseq');
  };
  by `clseq': egen long `cfwei2'=max(`cfwei') if `touse';
};
cap assert `cfwei'==`cfwei2' if `touse';
if _rc!=0 {;
  disp as error "Cluster frequency weights are not constant within clusters";
  error 498;
};
drop `cfwei2';

*
 Initialise tidotw, tidot, vidotw and vidot variables
 and tidotw and tidot variable lists,
 and set tempnames for stratum variables
*;
tempvar stratum vidotw vidot dvidot dtidot;
foreach X of new `vidotw' `vidot' `dtidot' {;
  qui gene double `X'=0 if `touse';
};
local tidotwlist "";
local tidotlist "";
forv i1=1(1)`nvar' {;
  tempvar tidotw`i1' tidot`i1';
  local tidotwlist "`tidotwlist' `tidotw`i1''";
  local tidotlist "`tidotlist' `tidot`i1''";
  qui gene double `tidotw`i1''=0 if `touse';
  qui gene double `tidot`i1''=0 if `touse';
};

*
 Calculate within-cluster vidots and tidots if required
 and update summary vidotw and tidotws
*;
if inlist("`funtype'","wcluster","bcluster","vonmises") {;
  qui {;
    gsort `bygp' `clseq' `wstrata', gene(`stratum');
    by `stratum': egen `dvidot' = total(`iwei') if `touse';
    replace `vidotw' = `vidotw' + `iwei'*`dvidot' if `touse';
    if "`tree'"!="notree" {;sort `stratum' `x';};
    forv i1=1(1)`nvar' {;
      local y: word `i1' of `varlist';
      local ycen: word `i1' of `cenindvars';
      mata: tidotforsomersd("`dtidot'","`touse'","`stratum'","`x'","`y'","`iwei'","`xcen'","`ycen'",`treeind');
      replace `tidotw`i1'' = `tidotw`i1'' + `iwei'*`dtidot' if `touse';
    };
    drop `stratum' `dvidot';
  };
};

*
 Calculate within-cluster within-B-stratum vidots and tidots if required
 and update summary vidotw and tidotws
*;
if ("`bstrata'"!="") & inlist("`funtype'","wcluster","bcluster","vonmises") {;
  qui {;
    if "`bstrata'"=="_n" {;
      gsort `bygp' `clseq' `wstrata' `_sortindex', gene(`stratum');
    };
    else {;
      gsort `bygp' `clseq' `wstrata' `bstrata', gene(`stratum');
    };
    by `stratum': egen `dvidot' = total(`iwei') if `touse';
    replace `vidotw' = `vidotw' - `iwei'*`dvidot' if `touse';
    if "`tree'"!="notree" {;sort `stratum' `x';};
    forv i1=1(1)`nvar' {;
      local y: word `i1' of `varlist';
      local ycen: word `i1' of `cenindvars';
      mata: tidotforsomersd("`dtidot'","`touse'","`stratum'","`x'","`y'","`iwei'","`xcen'","`ycen'",`treeind');
      replace `tidotw`i1'' = `tidotw`i1'' - `iwei'*`dtidot' if `touse';
    };
    drop `stratum' `dvidot';
  };
};

*
 Calculate within-W-stratum vidots and tidots if required
 and update summary vidots and tidots
*;
if inlist(`"`funtype'"',"vonmises","bcluster") {;
  qui {;
    gsort `bygp' `wstrata', gene(`stratum');
    by `stratum': egen `dvidot' = total(`prodwei') if `touse';
    replace `vidot' = `vidot' + `iwei'*`dvidot' if `touse';
    if "`tree'"!="notree" {;sort `stratum' `x';};
    forv i1=1(1)`nvar' {;
      local y: word `i1' of `varlist';
      local ycen: word `i1' of `cenindvars';
      mata: tidotforsomersd("`dtidot'","`touse'","`stratum'","`x'","`y'","`prodwei'","`xcen'","`ycen'",`treeind');
      replace `tidot`i1'' = `tidot`i1'' + `iwei'*`dtidot' if `touse';
    };
    drop `stratum' `dvidot';
  };
};

*
 Calculate within-W-stratum within-B-stratum vidots and tidots if required
 and update summary vidots and tidots
*;
if ("`bstrata'"!="") & inlist(`"`funtype'"',"vonmises","bcluster") {;
  qui {;
    if "`bstrata'"=="_n" {;
      gsort `bygp' `wstrata' `_sortindex', gene(`stratum');
    };
    else {;
      gsort `bygp' `wstrata' `bstrata', gene(`stratum');
    };
    by `stratum': egen `dvidot' = total(`prodwei') if `touse';
    replace `vidot' = `vidot' - `iwei'*`dvidot' if `touse';
    if "`tree'"!="notree" {;sort `stratum' `x';};
    forv i1=1(1)`nvar' {;
      local y: word `i1' of `varlist';
      local ycen: word `i1' of `cenindvars';
      mata: tidotforsomersd("`dtidot'","`touse'","`stratum'","`x'","`y'","`prodwei'","`xcen'","`ycen'",`treeind');
      replace `tidot`i1'' = `tidot`i1'' - `iwei'*`dtidot' if `touse';
    };
    drop `stratum' `dvidot';
  };
};

*
 Initialise estimation results that are the same
 for all by-groups
*;
* Estimates and covariance matrices (initialised to zero) *;
tempname b vce;
if "`taua'"=="" {;
  * Somers' D *;
  local param "somersd";local parmlab "Somers' D";
  bVzinit `nxvlist', b(`b') v(`vce');
};
else {;
  * Kendall's tau-a *;
  local param "taua";local parmlab "Kendall's tau-a";
  bVzinit `varlist', b(`b') v(`vce');
};
ereturn post `b' `vce', depname("`x'") properties("b V");
ereturn local cmd "somersd";
ereturn local param "`param'";
ereturn local parmlab "`parmlab'";
ereturn local tdist "`tdist'";
ereturn local depvar "`x'";
ereturn local clustvar "`cluster'";
ereturn local vcetype "Jackknife";
ereturn local wtype "`weight'";
ereturn local wexp "`exp'";
ereturn local cfweight `"`cfweight'"';
ereturn local funtype "`funtype'";
ereturn local wstrata "`wstrata'";
ereturn local bstrata "`bstrata'";
ereturn local predict "somers_p";

*
 Input vidots and tidots for each by-group
 and generate estimation results and possibly output
*;
sort `_byvars' `clseq' `_sortindex';
`bybyvars' _somersdby if `touse',
  namevars(`vidot' `varlist')
  uidotwvars(`vidotw' `tidotwlist') uidotvars(`vidot' `tidotlist')
  transf(`transf')
  clseq(`clseq') cfwei(`cfwei') ofwei(`ofwei')
  level(`level') `cimatrix';

*
 End of non-replay section (not indented)
*;
};

end;

program define _somersdby, byable(recall) eclass;
version 9.0;
*
 Process a by-group for somersd.
 (We can assume that the data are already sorted
 by by-group and cluster sequence number.)
*;

syntax [if] [,
  namevars(varlist numeric min=2)
  uidotwvars(varlist numeric min=2) uidotvars(varlist numeric min=2)
  TRansf(string)
  CLSeq(varname) CFWei(varname) OFWei(varname)
  Level(cilevel)
  CImatrix(passthru)
  ];
/*
namevars() indicates a list of variables to label the output matrices e(b) and e(V).
uidotwvars() indicates a list of variables containing vidotw and tidotws
  for input into the formulas for Somers' D and Kendall's tau-a.
  for which estimates, standard errors and confidence limits are calculated,
  and should have the same length as namevars.clseq() specifies a cluster sequence number variable.
uidotvars() indicates a list of variables containing vidot and tidots
  for input into the formulas for Somers' D and Kendall's tau-a.
  for which estimates, standard errors and confidence limits are calculated,
  and should have the same length as namevars.clseq() specifies a cluster sequence number variable.
cfwei() specifies cluster frequency weights,
  used if each cluster represents multiple identical clusters.
ofwei() specifies observation frequency weights
  used only for summing to derive a value for e(N).
level() specifies a confidence level.
cimatrix() specifies the name of a matrix to store the confidence limits
  (which may be asymmetric).
*/

marksample touse;

*
 Sum observation frequency weights
 (to be saved as e(N))
*;
qui summ `ofwei' if `touse', meanonly;
local nobs=r(sum);

*
 Abort if no observations
 (to be consistent with the rest of Stata)
*;
if `nobs'<=0 {;error 2000;};

*
 Create Mata-format string vectors namevec, uidotwvec and uidouvec
 from varlists namevars, uidotwvars and uidotvars, respectively
*;
local nuidotw: word count `uidotwvars';
local nuidot: word count `uidotvars';
local nname: word count `namevars';
if `nname'!=`nuidotw' | `nname'!=`nuidot' {;
  disp as error "Unequal lengths of uidotwvars, uidotvars and namevars";
  error 498;
};
local namecur: word 1 of `namevars';
local uidotwcur: word 1 of `uidotwvars';
local uidotcur: word 1 of `uidotvars';
local namevec `"("`namecur'""';
local uidotwvec `"("`uidotwcur'""';
local uidotvec `"("`uidotcur'""';
forv i1=2(1)`nname' {;
  local namecur: word `i1' of `namevars';
  local uidotwcur: word `i1' of `uidotwvars';
  local uidotcur: word `i1' of `uidotvars';
  local namevec `"`namevec',"`namecur'""';
  local uidotwvec `"`uidotwvec',"`uidotwcur'""';
  local uidotvec `"`uidotvec',"`uidotcur'""';
};
local namevec `"`namevec')"';
local uidotwvec `"`uidotwvec')"';
local uidotvec `"`uidotvec')"';

*
 Compute number of clusters, estimates and variance matrix
 for original parameters V and T(XY),
 whose ratios will be the Somers' D or Kendall's tau-a parameters
*;
if "`e(funtype)'"=="wcluster" {;
  mata: estvar1forsomersd("`touse'","`clseq'","`cfwei'",`namevec',`uidotwvec');
};
else if "`e(funtype)'"=="bcluster" {;
  mata: estvar1forsomersd("`touse'","`clseq'","`cfwei'",`namevec',`uidotwvec',`uidotvec',0);
};
else if "`e(funtype)'"=="vonmises" {;
  mata: estvar1forsomersd("`touse'","`clseq'","`cfwei'",`namevec',`uidotwvec',`uidotvec',1);
};
tempname nclust b vce;
scal `nclust'=r(N_clust);
matr def `b'=r(b);
matr def `vce'=r(V);

*
 Compute number of variables in the original varlist
*;
local nvar=colsof(`b')-1;
if `nvar'<1 {;
  disp as error "Original estimates matrix has too few columns";
  error 498;
};

*
 Convert V and T(XY) to Kendall's tau-a or Somers' D
 (setting matrices to zero if they are undefined,
 as Stata 6.0 does not permit missing values in matrices)
*;
if "`e(param)'"=="taua" {;
  * Kendall's tau-a *;
  tempname V invV invsqV transm transm1 transm2;
  matr `V'=`b'[1,1];scal `V'=trace(`V');
  if(`V'==0){;
    * Sum total of weights is zero, so set matrices to zero *;
    matr `vce'=0*`vce'[2...,2...];
    matr `b'=0*`b'[1,2...];
  };
  else{;
    * Some nonzero weights, so transform matrices *;
    scal `invV'=1/`V';scal `invsqV'=`invV'*`invV';
    matr `transm1'=(`b'[1,2...])';
    matr `transm1'=-`invsqV'*`transm1';
    matr `transm2'=`invV'*I(`nvar');
    matr `transm'=`transm1',`transm2';
    matr `vce'=`transm'*`vce'*`transm'';
    matr `vce'=0.5*(`vce'+`vce'');
    matr `b'=`invV'*`b'[1,2...];
  };
};
else{;
  * Somers' D *;
  tempname T invT invsqT transm transm0 transm1 transm2;
  matr `T'=`b'[1,2];scal `T'=trace(`T');
  if(`T'==0){;
    * All valid x-pairs are equal, so set matrices to zero *;
    matr `vce'=0*`vce'[3...,3...];
    matr `b'=0*`b'[1,3...];
  };
  else{;
    * Some valid unequal x-pairs, so transform matrices *;
    scal `invT'=1/`T';scal `invsqT'=`invT'*`invT';
    matr `transm1'=(`b'[1,3...])';
    matr `transm0'=0*`transm1';
    matr `transm1'=-`invsqT'*`transm1';
    matr `transm2'=`invT'*I(`nvar'-1);
    matr `transm'=`transm0',`transm1',`transm2';
    matr `vce'=`transm'*`vce'*`transm'';
    matr `vce'=0.5*(`vce'+`vce'');
    matr `b'=`invT'*`b'[1,3...];
  };
};

* Post estimation results *;
if("`e(tdist)'"==""){;
  *
   Normal distribution assumed
  *;
  ereturn repost b=`b' V=`vce', esample(`touse');
};
else{;
  *
   Student's t-distribution with df one less than number of clusters
  *;
  local nclustm=`nclust'-1;
  ereturn repost b=`b' V=`vce', esample(`touse');
  ereturn scalar df_r=`nclustm';
};
ereturn scalar N=`nobs';
ereturn scalar N_clust=`nclust';
ereturn local transf "";
ereturn local tranlab "";

* Transform if necessary *;
if "`transf'"=="" {;local transf="iden";};
capture corrtran, transf(`transf');
if _rc==0 {;
  matr  `b'=r(b);matr `vce'=r(V);
  ereturn repost b=`b' V=`vce';
  ereturn local transf "`r(transf)'";
  ereturn local tranlab "`r(tranlab)'";
};
else {;
  disp as error "Requested transformation transf(`transf') could not be performed";
  error 498;
};

*
 Display output
*;
_somersdplay, level(`level') `cimatrix';

end;

program define _somersdplay;
version 9.0;
*
 Display output
*;

syntax [,Level(cilevel) CImatrix(passthru)];
/*
level() specifies a confidence level.
cimatrix() specifies the name of a matrix to store the confidence limits
  (which may be asymmetric).
*/

* Check that confidence level is within range *;
if((`level'<10)|(`level'>99)){;
  disp as error "Level must be between 10 and 99 inclusive";
  exit 198;
};

* Functional type prefix *;
if "`e(funtype)'"=="wcluster" {;
  local funtypepref "Within-cluster ";
};
else if "`e(funtype)'"=="vonmises" {;
  local funtypepref "Von Mises ";
};

* CI label *;
local transf "`e(transf)'";
if(("`transf'"=="z")|("`transf'"=="asin")){;
  local cilab "Symmetric `level'% CI for transformed `e(parmlab)'";
};
else if("`transf'"=="zrho"){;
  local cilab "Symmetric `level'% CI for transformed Greiner's rho";
};
else if ("`transf'"=="c") | ("`transf'"=="roc") | ("`transf'"=="auroc") {;
  local cilab "Symmetric `level'% CI for Harrell's c";
};
else {;
  local cilab "Symmetric `level'% CI";
};

* Display symmetric CI *;
disp as text "`funtypepref'`e(parmlab)' with variable: " as result "`e(depvar)'";
disp as text "Transformation: " as result "`e(tranlab)'";
if "`e(wstrata)'"!="" {;
disp as text "Within strata defined by: " as result "`e(wstrata)'";
};
if "`e(bstrata)'"!="" {;
disp as text "Between strata defined by: " as result "`e(bstrata)'";
};
disp as text "Valid observations: " as result e(N);
if("`e(clustvar)'"!=""){;
  disp as text "Number of clusters: " as result e(N_clust);
};
if("`e(tdist)'"!=""){;
  disp as text "Degrees of freedom: " as result e(df_r);
};
disp as text _n "`cilab'";
ereturn display, level(`level');

* Back-transformed parameters if appropriate *;
parmtran, level(`level') `cimatrix';

end;

program define bVzinit;
version 9.0;
syntax varlist(min=1 max=9999) [, B(string) V(string) ];
*
 Take varlist as input
 and create, as output, matrices `b' and `v',
 with row and column names as for regression coefficients
 and covariance matrix respectively,
 but initialized to zero.
*;

local y:word 1 of `varlist';
local nvar:word count `varlist';

* Create "regression coefficient matrix" *;
if("`b'"!=""){;
  matr def `b'=J(1,`nvar',0);
  matr rownames `b'="y1";matrix colnames `b'=`varlist';
};
* Create "covariance matrix" *;
if("`v'"!=""){;
  matr `v'=J(`nvar',`nvar',0);
  matr rownames `v'=`varlist';matrix colnames `v'=`varlist';
};

end;

program define corrtran, rclass;
version 9.0;
*
 Transform a vector of correlation coefficients from e(b)
 and its dispersion matrix in e(V)
 using the transformation specified in transf
*;
syntax [, TRansf(string) ];

* Identify transformation (collapsing synonyms) *;
if(("`transf'"=="z")|("`transf'"=="harctan")|("`transf'"=="hatan")){;
  local transf="z";local tranlab="Fisher's z";
};
else if(("`transf'"=="arcsin")|("`transf'"=="arsin")
 |("`transf'"=="asin")){;
  local transf="asin";local tranlab="Daniels' arcsine";
};
else if(("`transf'"=="sinph")|("`transf'"=="rho")){;
  local transf="rho";local tranlab="Greiner's rho";
};
else if(("`transf'"=="zsinph")|("`transf'"=="zrho")){;
  local transf="zrho";local tranlab="z-transform of Greiner's rho";
};
else if ("`transf'"=="c") | ("`transf'"=="roc") | ("`transf'"=="auroc") {;
  local transf="c";local tranlab="Harrell's c";
};
else{;
  if(("`transf'"!="")&("`transf'"!="iden")){;
    disp as text "Note: unrecognized transformation - identity assumed";
  };
  local transf="iden";local tranlab="Untransformed";
};

* Abort if no estimation results *;
if("`e(cmd)'"==""){;
  disp as error "No estimation results present";
  error 301;
};

* Get matrices *;
tempname b vce;matr `b'=e(b);matrix `vce'=e(V);

*
 Set maximum absolute value for coefficients
 (because of danger of missing transformed values
 which cannot be inserted into Stata matrices)
*;
local tranlst="iden z asin rho zrho c";
tempname rmaxlst rmax;
matr def `rmaxlst'=(1,0.999999999999999,0.999999999999999,1,0.99999999,1);
matr colnames `rmaxlst'=`tranlst';
matr `rmax'=`rmaxlst'[1,"`transf'"];scal `rmax'=trace(`rmax');

local ncolb=colsof(`b');
* Initialize matrix of derivatives to zero *;
tempname dtran;matr `dtran'=0*`vce';
local i1=0;
while(`i1'<`ncolb'){;local i1=`i1'+1;
  tempname ri1 dri1;
  scal `ri1'=`b'[1,`i1'];
  * Set correlation to maximum if unity not allowed *;
  if(abs(`ri1')>`rmax'){;
    scal `ri1'=sign(`ri1')*`rmax';
  };
  * Calculate transformation and its derivative *;
  if("`transf'"=="z"){;
    scal `dri1'=1/(1-`ri1'*`ri1');
    scal `ri1'=0.5*(log(1+`ri1')-log(1-`ri1'));
  };
  else if("`transf'"=="asin"){;
    scal `dri1'=1/sqrt(1-`ri1'*`ri1');
    scal `ri1'=asin(`ri1');
  };
  else if("`transf'"=="rho"){;
    scal `dri1'=0.5*_pi*cos(0.5*_pi*`ri1');
    scal `ri1'=sin(0.5*_pi*`ri1');
  };
  else if("`transf'"=="zrho"){;
    * Apply sinph transform first *;
    scal `dri1'=0.5*_pi*cos(0.5*_pi*`ri1');
    scal `ri1'=sin(0.5*_pi*`ri1');
    * Then superimpose z-transform *;
    scal `dri1'=`dri1'/(1-`ri1'*`ri1');
    scal `ri1'=0.5*(log(1+`ri1')-log(1-`ri1'));
  };
  else if "`transf'"=="c" {;
    scal `dri1'=0.5;
    scal `ri1'=0.5*(`ri1'+1);
  };
  else{;
    * Identity transform *;
    scal `dri1'=1;
  };
  * Insert transformation results into matrices *;
  matr `dtran'[`i1',`i1']=`dri1';matr `b'[1,`i1']=`ri1';
};

* Transform dispersion matrix *;
if("`transf'"!="iden"){;matr `vce'=`dtran'*`vce'*`dtran';};

* Return values *;
return local transf "`transf'";
return local tranlab "`tranlab'";
return matrix b `b';
return matrix V `vce';

end;

program define parmtran;
version 9.0;
*
 Take confidence level and estimation output as input
 and create, as results output,
 a matrix containing asymmetric CIs for the untransformed parameters
 (which will be listed if appropriate)
*;
syntax [, Level(cilevel) CImatrix(string) ];
local transf="`e(transf)'";

* Create matrices containing estimates and standard errors *;
tempname btran sebtran se;
matr `btran'=(e(b))';matr `sebtran'=(vecdiag(e(V)))';
local nparam=rowsof(`btran');
local i1=0;
while(`i1'<`nparam'){;local i1=`i1'+1;
  scal `se'=`sebtran'[`i1',1];
  matr `sebtran'[`i1',1]=sqrt(`se');
};

* Define multiplier for creation of confidence intervals *;
tempname mult clfloat;
scal `clfloat'=`level'/100;
if("`e(tdist)'"!=""){;
  * Student's t-distribution *;
  local dof=e(df_r);
  * version 6: scal `mult'=invt(`dof',`clfloat');
  scal `mult'=invttail(`dof',0.5*(1-`clfloat'));
};
else{;
  * Normal distribution *;
  scal `mult'=invnormal(0.5*(1+`clfloat'));
};

* Create upper and lower confidence limits *;
tempname hwid min max ci;
matr `hwid'=`mult'*`sebtran';
matr `min'=`btran'-`hwid';
matr `max'=`btran'+`hwid';
matr `ci'=`btran',`min',`max';
if(("`transf'"=="rho")|("`transf'"=="zrho")){;
  matr colnames `ci'=Rho Minimum Maximum;
};
else if("`e(param)'"=="somersd"){;
  matr colnames `ci'=Somers_D Minimum Maximum;
};
else if("`e(param)'"=="taua"){;
  matr colnames `ci'=Tau_a Minimum Maximum;
};
else{;
  matr colnames `ci'=Estimate Minimum Maximum;
};

* Carry out inverse transformation if appropriate *;
local ncci=colsof(`ci');tempname r halfpi;
scal `halfpi'=_pi/2;
local i1=0;
while(`i1'<`nparam'){;local i1=`i1'+1;
  local i2=0;
  while(`i2'<`ncci'){;local i2=`i2'+1;
    scal `r'=`ci'[`i1',`i2'];
    if(("`transf'"=="z")|("`transf'"=="zrho")){;
      scal `r'=exp(2*`r');scal `r'=(`r'-1)/(`r'+1);
    };
    else if("`transf'"=="asin"){;
      * Convert out-of-range arcsines to + or - pi/2 *;
      if(`r'<-`halfpi'){;scal `r'=-`halfpi';};
      else if(`r'>`halfpi'){;scal `r'=`halfpi';};
      scal `r'=sin(`r');
    };
    matr `ci'[`i1',`i2']=`r';
  };
};  

* CI label *;
if(("`transf'"=="z")|("`transf'"=="asin")){;
  local cilab "Asymmetric `level'% CI for untransformed `e(parmlab)'";
};
else if("`transf'"=="zrho"){;
  local cilab "Asymmetric `level'% CI for untransformed Greiner's rho";
};
else{;
  local cilab "`level'% CI";
};

* Save confidence interval to output matrix if appropriate *;
if("`cimatrix'"!=""){;matr def `cimatrix'=`ci';};

* List confidence interval if appropriate *;
if(("`transf'"=="z")|("`transf'"=="asin")|("`transf'"=="zrho")){;
  disp as text _n "`cilab'";matr list `ci',noheader noblank;
};

end;
